!-------------------------------------------------------------------------------

!                      Code_Saturne version 3.0.0
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2013 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_extra_operations.f90
!>
!> \brief This function is called at the end of each time step, and has a very
!>  general purpose
!>  (i.e. anything that does not have another dedicated user subroutine)
!>
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     nbpmax        max. number of particles allowed
!> \param[in]     nvp           number of particle-defined variables
!> \param[in]     nvep          number of real particle properties
!> \param[in]     nivep         number of integer particle properties
!> \param[in]     ntersl        number of return coupling source terms
!> \param[in]     nvlsta        number of Lagrangian statistical variables
!> \param[in]     nvisbr        number of boundary statistics
!> \param[in]     itepa         integer particle attributes
!>                                (containing cell, ...)
!> \param[in]     dt            time step (per cell)
!> \param[in]     rtp, rtpa     calculated variables at cell centers
!>                               (at current and previous time steps)
!> \param[in]     propce        physical properties at cell centers
!> \param[in]     propfa        physical properties at interior face centers
!> \param[in]     propfb        physical properties at boundary face centers
!> \param[in]     ettp, ettpa   particle-defined variables
!> \param[in]                    (at current and previous time steps)
!> \param[in]     tepa          real particle properties
!> \param[in]                    (statistical weight, ...
!> \param[in]     statis        statistic means
!> \param[in]     stativ        accumulator for variance of volume statisitics
!> \param[in]     tslagr        Lagrangian return coupling term
!> \param[in]                    on carrier phase
!> \param[in]     parbor        particle interaction properties
!> \param[in]                    on boundary faces
!_______________________________________________________________________________


subroutine cs_user_extra_operations &
 ( nvar   , nscal  ,                                              &
   nbpmax , nvp    , nvep   , nivep  , ntersl , nvlsta , nvisbr , &
   itepa  ,                                                       &
   dt     , rtpa   , rtp    , propce , propfa , propfb ,          &
   ettp   , ettpa  , tepa   , statis , stativ , tslagr , parbor )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use lagpar
use lagran
use parall
use period
use ppppar
use ppthch
use ppincl
use field
use mesh
use user_module
!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          nbpmax , nvp    , nvep  , nivep
integer          ntersl , nvlsta , nvisbr

integer          itepa(nbpmax,nivep)

double precision dt(ncelet), rtp(ncelet,*), rtpa(ncelet,*)
double precision propce(ncelet,*)
double precision propfa(nfac,*), propfb(ndimfb,*)
double precision ettp(nbpmax,nvp) , ettpa(nbpmax,nvp)
double precision tepa(nbpmax,nvep)
double precision statis(ncelet,nvlsta), stativ(ncelet,nvlsta-1)
double precision tslagr(ncelet,ntersl)
double precision parbor(nfabor,nvisbr)

! Local variables
double precision , allocatable, dimension(:) :: ytmp
integer          ipcliu, ipcliv, ipcliw

! INSERT_VARIABLE_DEFINITIONS_HERE

!===============================================================================
integer impout,ii,icnt,npoint,jj,iel,iel1,irang1,irangv,iun

double precision ymax,xabs,yy,xyz(3),yy1,rrr,rrmin,xeps
double precision datave(15),xdudy
double precision, allocatable, dimension(:) :: averages,dudy
double precision, allocatable, dimension(:,:) :: warr,av_arr
double precision, allocatable, dimension(:,:,:) :: grad

double precision, dimension(:,:), pointer :: coefav
double precision, dimension(:,:,:), pointer :: coefbv
!--channel height
parameter(ymax=2.d0)
integer iccocg,idtcm,imom,inc,ipcmom,ntchrl
double precision::ttmp
integer iksgs,icksgs,inut,iDiss,ibet1,ibet4,itauijli,itauijnl,iFsgs,its,iSij,iSabs,icp1,ipcvst,ibscat
logical          ilved
!===============================================================================
! Initialization
!===============================================================================
ilved = .false.
ipcliu = iclrtp(iu,icoef)
ipcliv = iclrtp(iv,icoef)
ipcliw = iclrtp(iw,icoef)

allocate(grad(ncelet,3,3))
allocate(warr(ncelet,3))

!--writting frequency
ntchrl=16

!===============================================================================
! User operations
!===============================================================================

!--  Only the first time step --
!-------------------------------
if (ntcabs.eq.ntpabs+1) then 

   ! Find the points on y
   ! Save them to avoid doing it again
   !----------------------------------

   npoint = 400
   allocate(ytmp(npoint))
   ytmp = 0.d0


   nymax  = 0
   iel1   = -999
   irang1 = -999

   do ii = 1, npoint
      ! x-coordinate of the node
      xyz(1) = 0.d0
      ! y-coordinate of the node
      xyz(2) = float(ii-1)/float(npoint-1)*ymax
      ! z-coordinate of the node
      xyz(3) = 0.d0                
      call findpt(ncelet,ncel,xyzcen,xyz(1),xyz(2),xyz(3),iel,irangv)
      
      if ((iel.ne.iel1).or.(irangv.ne.irang1)) then ! If the last point found is the same as the new one
         iel1   = iel
         irang1 = irangv
         
         if (irangp.eq.irangv) then
            xabs = xyzcen(2,iel)
         else
            xabs = 0.d0
         endif
         if (irangp.ge.0) then
            iun = 1        
            call parbcr(irangv,iun,xabs) ! broadcast from rank irangv to all others
         endif
         if (irangp.le.0)then
            nymax = nymax +1
            ytmp(nymax) = xabs
         endif
      endif
   enddo

   if (irangp.ge.0) then
      ! -- broadcast nymax from rank 0 to all processors
      call parbci(0,1 ,nymax)
   endif
   write(nfecra,*) 'CELLS ON Y:',nymax
   ! --Allocate memories for user variables--
   !-----------------------------------------
   !--SGS MODEL definition
   !-- 0 : No SGS model
   !-- 1 : Dynamic Smagorinsky model
   !-- 2 : Explicit algerbaic model

   call init_user_module(ncel,ncelet,SGSMODEL)
   write(nfecra,*) 'Initialization'
   if (irangp.le.0) then
      do ii=1,nymax
         ydat(ii)=ytmp(ii)
      enddo
   endif

   if (irangp.ge.0) then
      ! -- broadcast ydat from rank 0 to all others
      call parbcr(0,nymax,ydat) 
   endif

   write(nfecra,*) 'Conectivity'

  ! Find conectivity between cells and y data.  

   invarc = 0
     
  ! For each cell, find which is the nearest ydat.
  do iel = 1, ncel
     rrr = 1.d13
     rrmin = 1.d13
     do ii = 1, nymax
        xeps = 1.d-8
        yy = ydat(ii)
        yy1 = xyzcen(2,iel)
        rrr = sqrt((yy-yy1)**2)
        if (rrr.lt.rrmin)then
           rrmin = rrr
           corrs(iel) = ii !--which y-coordinate corresponds to iel node
        endif
     enddo
     jj = corrs(iel)
     invarc(jj) = invarc(jj) + 1
  enddo
  ! Add the total number of cells
  do ii = 1,nymax
     jj = invarc(ii)
     ! -- global sum over jj
     if(irangp.ge.0) call parcpt(jj) 
     ! -- The total number of points that are related to each y
     invarc(ii) = jj  
     write(nfecra,*) 'invarc=',invarc(ii)
  enddo  

  deallocate(ytmp)
endif

allocate(dudy(nymax))

! ======================================================
! Plane averaging :
! ======================================================

if((mod(ntcabs,ntchrl).eq.0.and.ntchrl.ne.-1.and.ntcabs.ge.ntdmom(1)).or.ntcabs.eq.ntmabs)then
   write(nfecra,*) 'Plane averaging'
   allocate(averages(nymax))

   write(nfecra,*) 'Plane Averaging for',nymax, "cells"
   ! -- Loop on all the "moments" that contain averages
   do imom = 1, nbmomt
      write(nfecra,*) 'Averaging moment:',imom
      averages = 0.d0

      ipcmom = ipproc(icmome(imom))
      ! -- Add the variables to the plane averaged arrays
      do iel  = 1, ncel
         ii = corrs(iel)
         averages(ii) = averages(ii)+propce(iel,ipcmom)                        
      enddo
      ! -- Sum the averages on all processes
      if(irangp.ge.0)then
         !--Parallel sum over all processes
         call parrsm(nymax,averages)
      endif
      
      ! -- Obtain the averages by dividing by the number of cells on the streamwise & spanwise directions
      do ii = 1, nymax
         averages(ii) = averages(ii)/invarc(ii)
      enddo
      ! -- Add the averaged values to all cells to postprocess with ensight.
      do iel = 1, ncel
         ii = corrs(iel)
         propce(iel,ipcmom) = averages(ii)
      enddo
   enddo

   ! -- Calculate averaged gradients
   write(nfecra,*) 'Calculating the gradient of the averaged velocity'
   call flush(nfecra)

   warr=0.d0

   imom = 1
   ipcmom = ipproc(icmome(imom))
   idtcm = -idtmom(imom)
   
   do iel = 1,ncel
      warr(iel,1) = propce(iel,ipcmom)/max(dtcmom(idtcm),epzero)
   enddo

   if (irangp.ge.0.or.iperio.eq.1) call synsca(warr(1,1))
      
   iccocg=1
   inc=1
   write(nfecra,*) 'Boundary Condition'
   ! Boundary condition pointers for gradients and advection
   call field_get_coefa_v(ivarfl(iu), coefav)
   call field_get_coefb_v(ivarfl(iu), coefbv)
   

   ! - Compute gradient
   write(nfecra,*) 'Gradient'

   call grdvec &
        !==========
        ( iu  , imrgra , inc    , iccocg ,                    &
        nswrgr(iu) , imligr(iu) ,                             &
        iwarni(iu) , nfecra ,                                 &
        epsrgr(iu) , climgr(iu) , extrag(iu) ,                &
!        ilved  ,                                              &
        warr(1,1)       , coefav , coefbv ,                   &
        grad  )

   !  Write a 1D file with all moments
   impout = impusr(1)
   write(nfecra,*) 'Write averages'
   if (irangp.le.0) then
      open(impout,file="averages.dat")
      write(impout,*) '# y(m),<U>,<V>,<W>,<uu>,<vv>,<ww>,<uv>,<uw>,<vw>,<p>,<du/dy> '         
   endif

   iel1   = -999
   irang1 = -999
   do ii = 1, nymax
      xyz(1) = 0.d0
      xyz(2) = ydat(ii)
      xyz(3) = 0.d0
      call findpt(ncelet, ncel, xyzcen,xyz(1), xyz(2), xyz(3), iel, irangv)

      if ((iel.ne.iel1).or.(irangv.ne.irang1)) then
         iel1   = iel
         irang1 = irangv
         
         if (irangp.eq.irangv) then
            do imom = 1, nbmomt
               ipcmom = ipproc(icmome(imom))
               idtcm = -idtmom(imom)
               datave(imom) = propce(iel,ipcmom)/max(dtcmom(idtcm),epzero)               
            enddo
            xdudy = grad(iel,2,1)
         else
            do imom = 1, nbmomt
               datave(imom) = 0.d0
            enddo
            xdudy = 0.d0
         endif
         if (irangp.ge.0) then
            iun = nbmomt
            call parbcr(irangv,iun,datave(1:nbmomt))
            iun = 1
            call parbcr(irangv,iun,xdudy)
         endif
         if (irangp.le.0)then
            ! When writing, compute u'u' = <UU>-<U><U> and so on...
            ! assume the averages are as listed on the header.
            write(impout,1004) ydat(ii),(datave(jj), jj = 1, 3)     &
                 ,datave(4) - datave(1)**2                          &
                 ,datave(5) - datave(2)**2                          &
                 ,datave(6) - datave(3)**2                          &
                 ,datave(7) - datave(1)*datave(2)                   &
                 ,datave(8) - datave(1)*datave(3)                   &
                 ,datave(9) - datave(2)*datave(3)                   &
                 ,(datave(jj), jj = 10, nbmomt)                     &
                 , xdudy
         endif
      endif
   enddo
   if (irangp.le.0) close(impout)
   deallocate(averages)   
endif
write(nfecra,*) 'Second Gradient'
iccocg=1
inc=1
                                                       
! Boundary condition pointers for gradients and advection

call field_get_coefa_v(ivarfl(iu), coefav)
call field_get_coefb_v(ivarfl(iu), coefbv)

call grdvec &
     !==========
     ( iu  , imrgra , inc    , iccocg ,                    &
     nswrgr(iu) , imligr(iu) ,                             &
     iwarni(iu) , nfecra ,                                 &
     epsrgr(iu) , climgr(iu) , extrag(iu) ,                &
     !        ilved  ,                                              &
     rtpa(1,iu) , coefav , coefbv ,                        &
     grad  )


write(nfecra,*) 'Second Gradient finished'
dudy = 0.d0

! Add the variables to the plane averaged arrays
do iel  = 1, ncel
   ii = corrs(iel)
   dudy(ii) = dudy(ii)+grad(iel,2,1)        
enddo
! Sum the averages and the number of cells on all processors
if(irangp.ge.0) call parrsm(nymax,dudy)

! Obtain the averages by dividing by the number of cells on the streamwise & spanwise directions
do ii = 1, nymax
   dudy(ii) = dudy(ii)/invarc(ii)
enddo
write(nfecra,*) 'dudy finished'
write(nfecra,*) 'Time Series'
if (irangp.le.0) then
   impout = impusr(1)
   open(impout,file="TimeSeries.dat",access='append')
   write(impout,*) sqrt(abs((dudy(1)-dudy(nymax))/2.d0)*viscl0)/viscl0,ntcabs*dtref
   close (impout)
endif

if (SGSMODEL.eq.2) then
allocate(av_arr(nymax,31))

!--Add the variables to the plane averaged array
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 write(nfecra,*) 'Compuing my own averages'
 ipcvst = ipproc(ivisct)
 av_arr=0.d0
 do iel  = 1, ncel
    ii = corrs(iel)
    iksgs   =1
    icksgs  =2
    inut    =3
    idiss   =4
    itauijli=5
    itauijnl=11
    isij    =17
    isabs   =23
    ifsgs   =26
    ibet1   =27
    ibet4   =28
    its     =29
    icp1    =30
    ibscat  =31
    av_arr(ii,iksgs)=av_arr(ii,iksgs)+ksgs(iel)
    av_arr(ii,icksgs)=av_arr(ii,icksgs)+cksgs(iel)
    av_arr(ii,inut)=av_arr(ii,inut)+propce(iel,ipcvst)
    
    ttmp=(tauij(iel,1)+tauijnl(iel,1))*Sij(iel,1)+&
                                 2.d0*(tauij(iel,2)+tauijnl(iel,2))*Sij(iel,2)+&
                                 2.d0*(tauij(iel,3)+tauijnl(iel,3))*Sij(iel,3)+&
                                      (tauij(iel,4)+tauijnl(iel,4))*Sij(iel,4)+&
                                 2.d0*(tauij(iel,5)+tauijnl(iel,5))*Sij(iel,5)+&
                                      (tauij(iel,6)+tauijnl(iel,6))*Sij(iel,6)
    av_arr(ii,idiss)=av_arr(ii,idiss)+ttmp
    if (-ttmp.gt.0) then
       av_arr(ii,ibscat)=av_arr(ii,ibscat)-ttmp
     
    endif
    do jj=1,6
       av_arr(ii,itauijli+jj-1) =   av_arr(ii,itauijli+jj-1) +  tauij(iel,jj)
       av_arr(ii,itauijnl+jj-1) =   av_arr(ii,itauijnl+jj-1) +tauijnl(iel,jj)
       av_arr(ii,iSij+jj-1)     =   av_arr(ii,iSij+jj-1)     +    Sij(iel,jj)
    enddo
    av_arr(ii,iSabs) = av_arr(ii,iSabs)+   Sabs(iel)
    do jj=1,3
       av_arr(ii,iFsgs+jj-1) = av_arr(ii,iFsgs+jj-1)+Fsgs(iel,jj)
    enddo
    av_arr(ii,ibet1) = av_arr(ii,ibet1) +bet1(iel)
    av_arr(ii,ibet4) = av_arr(ii,ibet4) +bet4(iel)
    av_arr(ii,its) =   av_arr(ii,its)   +  ts(iel)
    av_arr(ii,icp1) =  av_arr(ii,icp1)  + cp1(iel)
 enddo
write(nfecra,*) 'parallel computation of my averages'
!--Sum the averages and the number of cells on all processors
 if(irangp.ge.0)then
    do ii = 1, nymax
       !--Ksgs
       xabs = av_arr(ii,iksgs)
       call parsom(xabs)
       av_arr(ii,iksgs) = xabs
       !--C_dyn
       xabs = av_arr(ii,icksgs)
       call parsom(xabs)
       av_arr(ii,icksgs) = xabs
       do jj=1,6
           !--tauij
           xabs = av_arr(ii,itauijli+jj-1)
           call parsom(xabs)
           av_arr(ii,itauijli+jj-1) = xabs
           !--tauijnl
           xabs = av_arr(ii,itauijnl+jj-1)
           call parsom(xabs)
           av_arr(ii,itauijnl+jj-1) = xabs
           !--Sij
           xabs = av_arr(ii,iSij+jj-1)
           call parsom(xabs)
           av_arr(ii,iSij+jj-1) = xabs
        enddo
        do jj=1,3
           !--FSGS
           xabs = av_arr(ii,iFsgs+jj-1)
           call parsom(xabs)
           av_arr(ii,iFsgs+jj-1) = xabs
        enddo
        !--Sabs
        xabs = av_arr(ii,iSabs)
        call parsom(xabs)
        av_arr(ii,iSabs) = xabs
        !--Diss
        xabs = av_arr(ii,iDiss)
        call parsom(xabs)
        av_arr(ii,iDiss) = xabs

        xabs = av_arr(ii,ibscat)
        call parsom(xabs)
        av_arr(ii,ibscat) = xabs
        !--Nut
        xabs = av_arr(ii,inut)
        call parsom(xabs)
        av_arr(ii,inut) = xabs
        !--bet1,bet4
        xabs = av_arr(ii,ibet1)
        call parsom(xabs)
        av_arr(ii,ibet1) = xabs
        xabs = av_arr(ii,ibet4)
        call parsom(xabs)
        av_arr(ii,ibet4) = xabs
        !--time scale
        xabs = av_arr(ii,its)
        call parsom(xabs)
        av_arr(ii,its) = xabs
        !-->Cp1 scale
        xabs = av_arr(ii,icp1)
        call parsom(xabs)
        av_arr(ii,icp1) = xabs
    enddo
 endif
 write(nfecra,*) 'putting averages in global variables'
 
 do ii = 1, nymax
    ksgsav(ii) =  ksgsav(ii)+dt(1)* av_arr(ii,iksgs)/invarc(ii)
    cksgsav(ii) = cksgsav(ii)+dt(1)*av_arr(ii,icksgs)/invarc(ii)

     do jj=1,6
          tauijav(ii,jj) =   tauijav(ii,jj)+dt(1)*  av_arr(ii,itauijli+jj-1)/invarc(ii)
        tauijnlav(ii,jj) = tauijnlav(ii,jj)+dt(1)*  av_arr(ii,itauijnl+jj-1)/invarc(ii)
            Sijav(ii,jj) =     Sijav(ii,jj)+dt(1)*  av_arr(ii,iSij+jj-1)/invarc(ii)
     enddo
     Sabsav(ii) = Sabsav(ii)+dt(1)*av_arr(ii,iSabs)/invarc(ii)
     Dissav(ii) = Dissav(ii)+dt(1)*av_arr(ii,iDiss)/invarc(ii)
     bscatav(ii) = bscatav(ii)+dt(1)*av_arr(ii,ibscat)/invarc(ii)
      Nutav(ii) =  Nutav(ii)+dt(1)*av_arr(ii,inut)/invarc(ii)

     do jj=1,3
        Fsgsav(ii,jj) = Fsgsav(ii,jj)+dt(1)*av_arr(ii,iFsgs+jj-1)/invarc(ii)
     enddo
     bet1av(ii) =  bet1av(ii)+dt(1)*av_arr(ii,ibet1)/invarc(ii)
     bet4av(ii) =  bet4av(ii)+dt(1)*av_arr(ii,ibet4)/invarc(ii)
       tsav(ii) =    tsav(ii)+dt(1)*av_arr(ii,its)  /invarc(ii)
      cp1av(ii) =   cp1av(ii)+dt(1)*av_arr(ii,icp1) /invarc(ii)
enddo

sumavdt=sumavdt+dt(1)
write(nfecra,*),'sumavdt=',dt(1),sumavdt
if(mod(ntcabs,16).eq.0.and.ntchrl.ne.-1.and.ntcabs.ge.ntdmom(1).or.ntcabs.eq.ntmabs.and.irangp.le.0) then
   write(nfecra,*) 'Writing my averages'
   call flush(nfecra)   
   open(impusr(1),file="KSGS.dat")
   open(impusr(2),file="tauijli.dat")
   open(impusr(3),file="tauijnl.dat")
   open(impusr(4),file="Fsgs.dat")
   open(impusr(5),file="Sij.dat")
   open(impusr(6),file="Dissnut.dat")
   open(impusr(7),file="beta1_4.dat")
   open(impusr(8),file="TimeScale.dat")
   open(impusr(9),file="Cp1.dat")
   do ii=1,nymax
      write(impusr(1),1005) ydat(ii),cksgsav(ii)/max(sumavdt,epzero),ksgsav(ii)/max(sumavdt,epzero)
      write(impusr(2),1005) ydat(ii),(tauijav(ii,jj)/max(sumavdt,epzero) , jj = 1, 6)
      write(impusr(3),1005) ydat(ii),(tauijnlav(ii,jj)/max(sumavdt,epzero) , jj = 1, 6)
      write(impusr(4),1005) ydat(ii),(Fsgsav(ii,jj)/max(sumavdt,epzero), jj = 1, 3)
      write(impusr(5),1005) ydat(ii),(Sijav(ii,jj)/max(sumavdt,epzero), jj = 1, 6),Sabsav(ii)/max(sumavdt,epzero)
      
      write(impusr(6),1005) ydat(ii),Dissav(ii)/max(sumavdt,epzero),nutav(ii)/max(sumavdt,epzero),bscatav(ii)/max(sumavdt,epzero)
      write(impusr(7),1005) ydat(ii),bet1av(ii)/max(sumavdt,epzero),bet4av(ii)/max(sumavdt,epzero)
      write(impusr(8),1005) ydat(ii),tsav(ii)/max(sumavdt,epzero)
      write(impusr(9),1005) ydat(ii),Cp1av(ii)/max(sumavdt,epzero)
   enddo
   do ii=1,9
      close(impusr(ii))
   enddo
endif
deallocate(av_arr)
endif

!--------
! Formats
!--------

1001 format('WARNING:  TOTAL NUMBER CELLS SHOULD BE',I5,/,       &
          'FOUND:',I5, 1E17.9)
1002 format(i3.3)
1003 format(2e17.9)
1004 format(20e17.9)
1005 format(20e17.9)

! Free memory
deallocate(grad)
deallocate(warr)
deallocate(dudy)

!----
! End
!----
return
end subroutine cs_user_extra_operations
