!-------------------------------------------------------------------------------

!                      Code_Saturne version 3.0.0
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2013 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_physical_properties.f90
!> \brief Definition of physical variable laws.
!>
!> usphyv
!> \brief Definition of physical variable laws.
!>
!> \section Warning
!>
!> It is \b forbidden to modify turbulent viscosity \c visct here
!> (a specific subroutine is dedicated to that: \ref usvist)
!>
!> - icp = 1 must <b> have been specified </b>
!>    in \ref usipph if we wish to define a variable specific heat
!>    cp (otherwise: memory overwrite).
!>
!> - ivisls = 1 must <b> have been specified </b>
!>    in \ref usipsc if we wish to define a variable viscosity
!>    \c viscls (otherwise: memory overwrite).
!>
!>
!> \remarks
!>  - This routine is called at the beginning of each time step
!>    Thus, <b> AT THE FIRST TIME STEP </b> (non-restart case), the only
!>    values initialized before this call are those defined
!>      - in the GUI or  \ref usipsu (cs_user_parameters.f90)
!>             - density    (initialized at \c ro0)
!>             - viscosity  (initialized at \c viscl0)
!>      - in the GUI or \ref cs_user_initialization
!>             - calculation variables (initialized at 0 by defaut
!>             or to the value given in the GUI or in \ref cs_user_initialization)
!>
!>  - We may define here variation laws for cell properties, for:
!>     - density:                                    rom    kg/m3
!>     - density at boundary faces:                  romb   kg/m3)
!>     - molecular viscosity:                        viscl  kg/(m s)
!>     - specific heat:                              cp     J/(kg degrees)
!>     - diffusivities associated with scalars:      visls kg/(m s)
!>
!> \b Warning: if the scalar is the temperature, visls corresponds
!> to its conductivity (Lambda) in W/(m K)
!>
!>
!> The types of boundary faces at the previous time step are available
!>   (except at the first time step, where arrays \c itypfb and \c itrifb have
!>   not been initialized yet)
!>
!> It is recommended to keep only the minimum necessary in this file
!>   (i.e. remove all unused example code)
!>
!>
!> \section cell_id Cells identification
!>
!> Cells may be identified using the \ref getcel subroutine.
!> The syntax of this subroutine is described in the
!> \ref cs_user_boundary_conditions subroutine,
!> but a more thorough description can be found in the user guide.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     ibrom         indicator of filling of romb array
!> \param[in]     dt            time step (per cell)
!> \param[in]     rtp, rtpa     calculated variables at cell centers
!> \param[in]                    (at current and previous time steps)
!> \param[in]     propce        physical properties at cell centers
!> \param[in]     propfa        physical properties at interior face centers
!> \param[in]     propfb        physical properties at boundary face centers
!_______________________________________________________________________________


!===============================================================================

!===============================================================================
! Purpose:
! -------

!> usvist
!> \brief Modify turbulent viscosity
!>
!> This subroutine is called at beginning of each time step
!> after the computation of the turbulent viscosity
!> (physical quantities have already been computed in \ref usphyv).
!>
!> Turbulent viscosity \f$ \mu_T \f$ (kg/(m s)) can be modified.
!>
!> A modification of the turbulent viscosity can lead to very
!> significant differences betwwen solutions and even give wrong
!> results.
!>
!> This subroutine is therefore reserved to expert users.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     ncepdp        number of cells with head loss
!> \param[in]     ncesmp        number of cells with mass source term
!> \param[in]     icepdc        head loss cell numbering
!> \param[in]     icetsm        numbering of cells with mass source term
!> \param[in]     itypsm        kind of mass source for each variable
!>                               (cf. \ref ustsma)
!> \param[in]     dt            time step (per cell)
!> \param[in]     rtp, rtpa     calculated variables at cell centers
!>                               (at current and previous time steps)
!> \param[in,out] propce        physical properties at cell centers
!> \param[in]     propfa        physical properties at interior face centers
!> \param[in]     propfb        physical properties at boundary face centers
!> \param[in]     ckupdc        work array for head loss terms
!> \param[in]     smacel        values of variables related to mass source
!>                              term. If ivar=ipr, smacel=mass flux
!_______________________________________________________________________________

subroutine usvist &
 ( nvar   , nscal  , ncepdp , ncesmp ,                            &
   icepdc , icetsm , itypsm ,                                     &
   dt     , rtp    , rtpa   , propce , propfa , propfb ,          &
   ckupdc , smacel )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use numvar
use optcal
use cstphy
use entsor
use parall
use period
use mesh
use field
use user_module
!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          ncepdp , ncesmp

integer          icepdc(ncepdp)
integer          icetsm(ncesmp), itypsm(ncesmp,nvar)

double precision dt(ncelet), rtp(ncelet,*), rtpa(ncelet,*)
double precision propce(ncelet,*)
double precision propfa(nfac,*), propfb(ndimfb,*)
double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)

! Local variables

integer          iel, iccocg, inc
integer          ipcrom, ipcvst
double precision dudx, dudy, dudz, sqdu, visct, rom

double precision, dimension(:,:), pointer :: coefav
double precision, dimension(:,:,:), pointer :: coefbv
double precision, allocatable, dimension(:,:,:) :: gradv
!===============================================================================
integer::ii
integer :: err = 0
!--Explicit algebraic model
double precision oij(ncelet,6),oijf(ncelet,6),Sabsf(ncelet),oabs,oabsf,som(6)
double precision Delta,Deltaf,xfil,xfil2,xa,xb,cdamp

double precision nut(ncelet),ff,p1,p2,ss,c1,oo
double precision cnum,cdenum

double precision w1(ncelet),w2(ncelet),w3(ncelet),w4(ncelet)
double precision w5(ncelet),w6(ncelet),w7(ncelet),w8(ncelet)

!--EASTOCHASTIC STUFF--
double precision tstoch,bstoch,stoch,Diss_stoch,rr
logical EASTOCH,PEPS

!===============================================================================
!--indicator
ipcvst = ipproc(ivisct)

if (SGSMODEL.eq.0) then
   !--No SGS model
   do iel = 1, ncel      
      propce(iel,ipcvst) = 0.d0      
   enddo
   return
elseif(SGSMODEL.eq.1) then
   !--Dynamic Smagorinsky model
   return
else
!--Explicit algebraic model
endif
!===============================================================================
!--Explicit algebraic model
!===============================================================================
!===============================================================================
! 1.2 Initialization of the parameters
!===============================================================================
allocate(gradv(ncelet,3,3))
!--Grid-filter size
xfil   = 1.0d0!--xlesfl
!--Test-filter size
xfil2  = 3.0d0!--xlesfd

xa     = ales
xb     = bles

ff=3.d0/5.d0
!--------------------------------------------------
!--Solve for P/EPS(PEPS=true) or not (PEPS=false)
!--------------------------------------------------
!PEPS=.true.
 PEPS=.false.
!-----------------
! Stochastic tsuff
!-----------------
EASTOCH=.false.
if (EASTOCH) then
   bstoch=2.0
   stoch=1.d0
else
   stoch=0.d0
endif

if (ntcabs.eq.ntpabs+1) then ! Only the first time step
   allocate(bet1(ncelet))
   allocate(bet4(ncelet))
   allocate(ksgs(ncelet))
   allocate(cksgs(ncelet))
   allocate(Diss(ncelet))
   allocate(bscatav(ncelet))
   allocate(Fsgs(ncelet,3))
   allocate(tauij(ncelet,6))
   allocate(tauijnl(ncelet,6))
   allocate(Sij(ncelet,6))
   allocate(Sabs(ncelet))
   allocate(ts(ncelet))
   allocate(cp1(ncelet))
   allocate(volumef(ncelet))
   allocate(gstor(ncelet))
  
   call cfiltr(volume,volumef,w4, w5)
   volumef=volume
   call zufalli(abs((xyzcen(1,10))))
   gstor=0.0
endif

!===============================================================================
! 1.2 Compute Sij and Oij
!===============================================================================
iccocg=1
inc=1
! Boundary condition pointers for gradients and advection
call field_get_coefa_v(ivarfl(iu), coefav)
call field_get_coefb_v(ivarfl(iu), coefbv)

! - Compute gradient of u 

call grdvec &
     !==========
     ( iu  , imrgra , inc    , iccocg ,                    &
     nswrgr(iu) , imligr(iu) ,                             &
     iwarni(iu) , nfecra ,                                 &
     epsrgr(iu) , climgr(iu) , extrag(iu) ,                &
     rtpa(1,iu) , coefav , coefbv ,                        &
     gradv  )

!--compute Sij and Oij
do iel = 1, ncel
   Sij(iel,1)=       gradv(iel,1,1)  !--S11
   Sij(iel,2)= 0.5d0*gradv(iel,2,1)  !--S12
   Sij(iel,3)= 0.5d0*gradv(iel,3,1)  !--S13

   Oij(iel,1)=       0.d0            !--O11
   Oij(iel,2)= 0.5d0*gradv(iel,2,1)  !--S12
   Oij(iel,3)= 0.5d0*gradv(iel,3,1)  !--S13

enddo


!--compute Sij and Oij
do iel = 1, ncel
   Sij(iel,2)= Sij(iel,2)+0.5d0*gradv(iel,1,2)  !--S12
   Sij(iel,4)=                  gradv(iel,2,2)  !--S22
   Sij(iel,5)=            0.5d0*gradv(iel,3,2)  !--S23

   Oij(iel,2)= Oij(iel,2)-0.5d0*gradv(iel,1,2)  !--O12
   Oij(iel,4)=            0.d0                  !--O22
   Oij(iel,5)=            0.5d0*gradv(iel,3,2)  !--O23

enddo

!--compute Sij and Oij
do iel = 1, ncel
   Sij(iel,3)= Sij(iel,3)+0.5d0*gradv(iel,1,3)  !--S13
   Sij(iel,5)= Sij(iel,5)+0.5d0*gradv(iel,2,3)  !--S23
   Sij(iel,6)=                  gradv(iel,3,3)  !--S33

   Oij(iel,3)= Oij(iel,3)-0.5d0*gradv(iel,1,3)  !--O13
   Oij(iel,5)= Oij(iel,5)-0.5d0*gradv(iel,2,3)  !--O23
   Oij(iel,6)=             0.d0                 !--O33
   sabs(iel)=sqrt(2.d0*(Sij(iel,1)**2.+Sij(iel,4)**2.+Sij(iel,6)**2.)+&
                  4.d0*(Sij(iel,2)**2.+Sij(iel,3)**2.+Sij(iel,5)**2.))

enddo


!--Compute the filtere strain rate tensor
call cfiltr(Sabs, Sabsf, w4, w5)

!===============================================================================
! 1.3 Computation of the KSGS
!===============================================================================
do iel = 1, ncel
   !-- w1=u^2+v^2+w^2
   w1(iel)=rtpa(iel,iu)**2+rtpa(iel,iv)**2+rtpa(iel,iw)**2  
enddo
call cfiltr( w1           , w6     , w4     , w5     )
    
call cfiltr( rtpa(1,iu)   , w2     , w1     , w5     )

call cfiltr( rtpa(1,iv)   , w3     , w1     , w5     )

call cfiltr( rtpa(1,iw)   , w4     , w1     , w5     )
     


do iel = 1, ncel
   !-- w6=(u^2+v^2+w^2)-filter(u^2+v^2+w^2)
   w6(iel) = w6(iel)-(w2(iel)**2+w3(iel)**2+w4(iel)**2)
   !-- Delta=xfil*(xa*volume(iel))**xb
   !--       2.0   1.0            0.33
   delta   = xfil *(xa*volumef(iel))**xb
   !-- Delta^2|Sij|^2
   w2(iel) = delta**2*sabs(iel)**2  
enddo

call cfiltr( w2     , w3     , w4     , w5     )

do iel = 1, ncel
   delta  = xfil *(xa*volumef(iel))**xb
   !-- xfil2*xfil*(xa*volume(iel))**xb, xfil2=1.5
   deltaf = xfil2 *delta
   !-- Deltaf^2|filter(Sij)|^2
   w5(iel)= deltaf**2*Sabsf(iel)**2
   
   !--Numerator
   w4(iel)= w6(iel)!*(w5(iel)-w3(iel))
   !--Denuminator
   w5(iel)=(w5(iel)-w3(iel))!**2.
enddo

!--Parallel communication of the denuminator

!--local averaging of the denuminator for stabilizing
call cfiltr( w5     , w3     , w6     , w8     )
     
!--local averaging of the numerator for stabilizing
call cfiltr( w4     , w7     , w6     , w8     )
     
do iel = 1, ncel
   !- The dynamic coefficient for Ksgs
   w4(iel) = max(0.5d0*w7(iel) / w3(iel),0.d0)
enddo

cksgs=w4    

!--Compute the SGS stresses

do iel = 1, ncel
   !-- xfil*(xa*volume(iel))**xb

   delta = xfil*(xa*volumef(iel))**xb
   deltaf = xfil2*delta

   !-- Ksgs=c*Delta^2|Sij|^2 
   !-- [Eq. 2.15 JFM (2009), Vol.639,p. 407]
   Ksgs(iel)=cksgs(iel)*delta**2*Sabs(iel)**2

   !-- [Eq. 2.19 JFM (2009), Vol.639,p. 408]
   ts(iel) =max(12.5*sqrt(cksgs(iel))/Sabs(iel),1.e-23)
   oabs=-ts(iel)**2.*2.d0*(oij(iel,2)**2+oij(iel,3)**2+oij(iel,5)**2)

   !-- [Eq. 5.6 JFM (2009), Vol.639,p. 415]
   if (PEPS) then
      ts(iel)=max(14.5*sqrt(cksgs(iel))/Sabs(iel),1.e-23)
      oabs=-ts(iel)**2.*2.d0*(oij(iel,2)**2+oij(iel,3)**2+oij(iel,5)**2)
      oo=oabs
      cp1(iel)=9./4.*(min(max(4./9.0*50.0*(cksgs(iel)/0.2)**1.25,1.0),1.8)-1.)                

      ss= ts(iel)**2*( sij(iel,1)**2+2.d0*sij(iel,2)**2+&
                  2.d0*sij(iel,3)**2+     sij(iel,4)**2+&
                  2.d0*sij(iel,5)**2+     sij(iel,6)**2)
            
      p1=(cp1(iel)**2./27.d0+(4.d0/3.d0-ff)*(9./4.)**2./6.*ss-2.d0/3.d0*oo)*cp1(iel)
      p2=p1**2.-(cp1(iel)**2./9.d0+(4.d0/3.d0-ff)*(9.d0/4.d0)**2./3.d0*ss+2.d0/3.d0*oo)**3.
            
      if (p2.ge.0) then
         c1=cp1(iel)/3.+&
           sign((abs(p1+sqrt(p2)))**(1./3.),&
                     p1+sqrt(p2))+&
           sign((abs(p1-sqrt(p2)))**(1./3.),&
                     p1-sqrt(p2))
      else
         if (p1.ge.0) then
            c1=cp1(iel)/3.d0+2.d0*(p1**2.-p2)**(1.d0/6.d0)&
              *cos(acos(p1/sqrt(p1**2.-p2))/3.d0)
        else
            c1=cp1(iel)/3.d0+2.d0*(p1**2.-p2)**(1.d0/6.d0)&
                 *cos(-acos(-p1/sqrt(p1**2.-p2)/3.d0)&
                 +3.141592653589793/3.d0)
         endif
      endif
      cp1(iel)=max(c1,0.d0)
   else
      cp1(iel) =max(50.d0*(cksgs(iel)/0.2d0)**1.25,4.d0/9.d0) !-- Cp1 is already multiplied by (9./4.)
   endif
   !-- [Eq. 2.14 JFM (2009), Vol.639,p. 407]
   bet4(iel)=-(4./3.-ff)*9./4./(cp1(iel)**2.-2.d0*oabs)
   bet1(iel)= cp1(iel)*bet4(iel)


   if (EASTOCH) then
      Diss_stoch=max(abs(-.5*ts(iel)*ksgs(iel)*bet1(iel)*sabs(iel)**2.),1.e-6)
      tstoch= max(0.1*(delta**2.0/Diss_stoch)**(1./3.),dtref)
      call normalen(1,rr)
      gstor(iel)=(1-dtref/tstoch)*gstor(iel)+bstoch*sqrt(2.*dtref/tstoch)*rr
   endif
   !-- Sik Okj-Oik Skj
   som(1)= -2.*(Sij(iel,2)*Oij(iel,2)+Sij(iel,3)*Oij(iel,3))
      
   som(2)=Sij(iel,1)*Oij(iel,2)-&
          Sij(iel,3)*Oij(iel,5)-&
          Sij(iel,4)*Oij(iel,2)-& 
          Sij(iel,5)*Oij(iel,3)

   som(3)=Sij(iel,1)*Oij(iel,3)+& 
          Sij(iel,2)*Oij(iel,5)-& 
          Sij(iel,5)*Oij(iel,2)-& 
          Sij(iel,6)*Oij(iel,3)

   som(4)=2.d0*(Oij(iel,2)*Sij(iel,2)-Oij(iel,5)*Sij(iel,5))
                   
   som(5)=Oij(iel,3)*Sij(iel,2)+& 
          Oij(iel,5)*Sij(iel,4)+& 
          Oij(iel,2)*Sij(iel,3)-&
          Oij(iel,5)*Sij(iel,6)
   
   som(6)=2.d0*(Oij(iel,3)*Sij(iel,3)+Sij(iel,5)*Sij(iel,5))
           
   !--1st and third terms in [Eq. 2.13 JFM (2009), Vol.639,p. 407]

   tauijnl(iel,1)=Ksgs(iel)*(2.d0/3.d0+bet4(iel)*ts(iel)**2*som(1))   
   tauijnl(iel,2)=Ksgs(iel)*(          bet4(iel)*ts(iel)**2*som(2))   
   tauijnl(iel,3)=Ksgs(iel)*(          bet4(iel)*ts(iel)**2*som(3))   
   tauijnl(iel,4)=Ksgs(iel)*(2.d0/3.d0+bet4(iel)*ts(iel)**2*som(4))   
   tauijnl(iel,5)=Ksgs(iel)*(          bet4(iel)*ts(iel)**2*som(5))   
   tauijnl(iel,6)=Ksgs(iel)*(2.d0/3.d0+bet4(iel)*ts(iel)**2*som(6))
   
   !-Eddy viscosity computed from the EASSM
   nut(iel)=-bet1(iel)*ts(iel)*Ksgs(iel)/2.d0*(1+gstor(iel)*stoch)

   tauij(iel,1)=-2.0*nut(iel)*Sij(iel,1)
   tauij(iel,2)=-2.0*nut(iel)*Sij(iel,2)
   tauij(iel,3)=-2.0*nut(iel)*Sij(iel,3)
   tauij(iel,4)=-2.0*nut(iel)*Sij(iel,4)
   tauij(iel,5)=-2.0*nut(iel)*Sij(iel,5)
   tauij(iel,6)=-2.0*nut(iel)*Sij(iel,6)

enddo

Fsgs=0.d0

!-imrgra: Type of gradient reconstruction
!-
!-nswrgr() (<=1 gradient not reconstructed )
!-imligr() type of gradient limitation (-1 no limit, 0 based on the neighbours 1 superior order)
!-iphydp 
!-iwarni
!-epsrgr   relative precision of the iterative recon.
!-climgr() factor of gradient limitation (real >0.)
!-extrag()
!-coefa (,)boundary conditions
!-coefb (,)boundary conditions
!-extrag()

!   write(nfecra,*) imrgra,nswrgr(iuiph),imligr(iuiph),epsrgr(iuiph),extrag(iuiph),climgr(iuiph)
!                   0     ,   100       ,   -1        , 1.0E-005   , 0.0, 1.5

iccocg=1
inc=1
! Boundary condition pointers for gradients and advection
call field_get_coefa_v(ivarfl(iu), coefav)
call field_get_coefb_v(ivarfl(iu), coefbv)

! - Compute gradient of

call grdvec &
     !==========
     ( iu   , imrgra , inc    , iccocg ,                    &
     nswrgr(iu)  , imligr(iu) ,                             &
     iwarni(iu)  , nfecra ,                                 &
     epsrgr(iu)  , climgr(iu) , extrag(iu) ,                &
     tauijnl(1,1), coefav , coefbv ,                        &
     gradv  )

do iel = 1, ncel
   Fsgs(iel,1)=Fsgs(iel,1)+gradv(iel,1,1)
enddo

! - Compute gradient of

call grdvec &
     !==========
     ( iu   , imrgra , inc    , iccocg ,                    &
     nswrgr(iu)  , imligr(iu) ,                             &
     iwarni(iu)  , nfecra ,                                 &
     epsrgr(iu)  , climgr(iu) , extrag(iu) ,                &
     tauijnl(1,2), coefav , coefbv ,                        &
     gradv  )

do iel = 1, ncel
   Fsgs(iel,1)=Fsgs(iel,1)+gradv(iel,2,1)
   Fsgs(iel,2)=Fsgs(iel,2)+gradv(iel,1,1)
enddo

call grdvec &
     !==========
     ( iu   , imrgra , inc    , iccocg ,                    &
     nswrgr(iu)  , imligr(iu) ,                             &
     iwarni(iu)  , nfecra ,                                 &
     epsrgr(iu)  , climgr(iu) , extrag(iu) ,                &
     tauijnl(1,3), coefav , coefbv ,                        &
     gradv  )

do iel = 1, ncel
   Fsgs(iel,1)=Fsgs(iel,1)+gradv(iel,3,1)
   Fsgs(iel,3)=Fsgs(iel,3)+gradv(iel,1,1)
enddo

call grdvec &
     !==========
     ( iu   , imrgra , inc    , iccocg ,                    &
     nswrgr(iu)  , imligr(iu) ,                             &
     iwarni(iu)  , nfecra ,                                 &
     epsrgr(iu)  , climgr(iu) , extrag(iu) ,                &
     tauijnl(1,4), coefav , coefbv ,                        &
     gradv  )

do iel = 1, ncel
   Fsgs(iel,2)=Fsgs(iel,2)+gradv(iel,2,1)
enddo

call grdvec &
     !==========
     ( iu   , imrgra , inc    , iccocg ,                    &
     nswrgr(iu)  , imligr(iu) ,                             &
     iwarni(iu)  , nfecra ,                                 &
     epsrgr(iu)  , climgr(iu) , extrag(iu) ,                &
     tauijnl(1,5), coefav , coefbv ,                        &
     gradv  )

do iel = 1, ncel
   Fsgs(iel,2)=Fsgs(iel,2)+gradv(iel,3,1)
   Fsgs(iel,3)=Fsgs(iel,3)+gradv(iel,2,1)   
enddo

call grdvec &
     !==========
     ( iu   , imrgra , inc    , iccocg ,                    &
     nswrgr(iu)  , imligr(iu) ,                             &
     iwarni(iu)  , nfecra ,                                 &
     epsrgr(iu)  , climgr(iu) , extrag(iu) ,                &
     tauijnl(1,6), coefav , coefbv ,                        &
     gradv  )

do iel = 1, ncel
   Fsgs(iel,3)=Fsgs(iel,3)+gradv(iel,3,1)
enddo

do iel=1,ncel
   propce(iel,ipcvst)=nut(iel)
enddo
! Free memory
deallocate(gradv)

!--------
! Formats
!--------

!----
! End
!----

return
end subroutine usvist
!===============================================================================
