!===============================================================================
! User source terms definition.
!
! 1) Momentum equation (segegrated solver)
! 2) Momentum equation (coupled solver)
! 3) Species transport
! 4) Turbulence (k-epsilon, k-omega, Rij-epsilon, v2-f, Spalart-Allmaras)
!===============================================================================

!-------------------------------------------------------------------------------

!VERS

! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2013 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_source_terms.f90
!>
!> \brief Additional right-hand side source terms
!>
!> \brief Additional right-hand side source terms for velocity components equation
!> (Navier-Stokes)
!>
!> \section use  Usage
!>
!> The routine is called if the coulped solving of the velocity components is
!> turned on (\c ivelco=1).
!>
!> The additional source term is decomposed into an explicit part (\c crvexp) and
!> an implicit part (\c crvimp) that must be provided here.
!> The resulting equation solved by the code for a velocity is:
!> \f[
!>  \rho \norm{\vol{\celli}} \DP{\vect{u}} + ....
!>   = \tens{crvimp} \vect{u} + \vect{crvexp}
!> \f]
!>
!> Note that \c crvexp and \c crvimp are defined after the Finite Volume integration
!> over the cells, so they include the "volume" term. More precisely:
!>   - crvexp is expressed in kg.m/s2
!>   - crvimp is expressed in kg/s
!>
!> The \c crvexp and \c crvimp arrays are already initialized to 0
!> before entering the
!> the routine. It is not needed to do it in the routine (waste of CPU time).
!>
!> For stability reasons, Code_Saturne will not add -crvimp directly to the
!> diagonal of the matrix, but Max(-crvimp,0). This way, the crvimp term is
!> treated implicitely only if it strengthens the diagonal of the matrix.
!> However, when using the second-order in time scheme, this limitation cannot
!> be done anymore and -crvimp is added directly. The user should therefore test
!> the negativity of crvimp by himself.
!>
!> When using the second-order in time scheme, one should supply:
!>   - crvexp at time n
!>   - crvimp at time n+1/2
!>
!> The selection of cells where to apply the source terms is based on a
!> \ref getcel command. For more info on the syntax of the \ref getcel command,
!> refer to the user manual or to the comments on the similar command
!> \ref getfbr in the routine \ref cs_user_boundary_conditions.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     ncepdp        number of cells with head loss terms
!> \param[in]     ncesmp        number of cells with mass source terms
!> \param[in]     ivar          index number of the current variable
!> \param[in]     icepdc        index number of cells with head loss terms
!> \param[in]     icetsm        index number of cells with mass source terms
!> \param[in]     itypsm        type of mass source term for each variable
!>                               (see \ref ustsma)
!> \param[in]     dt            time step (per cell)
!> \param[in]     rtpa          calculated variables at cell centers
!>                               (preceding time steps)
!> \param[in]     propce        physical properties at cell centers
!> \param[in]     propfa        physical properties at interior face centers
!> \param[in]     propfb        physical properties at boundary face centers
!> \param[in]     ckupdc        head loss coefficient
!> \param[in]     smacel        value associated to each variable in the mass
!>                               source terms or mass rate (see \ref ustsma)
!> \param[out]    crvexp        explicit part of the source term
!> \param[out]    crvimp        implicit part of the source term
!_______________________________________________________________________________

subroutine ustsnv &
 ( nvar   , nscal  , ncepdp , ncesmp ,                            &
   ivar   ,                                                       &
   icepdc , icetsm , itypsm ,                                     &
   dt     , rtpa   , propce , propfa , propfb ,                   &
   ckupdc , smacel ,                                              &
   crvexp , crvimp )

!===============================================================================

!===============================================================================
  ! Module files
!===============================================================================

use paramx
use numvar
use entsor
use optcal
use cstphy
use parall
use period
use mesh
use user_module
!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          ncepdp , ncesmp
integer          ivar

integer          icepdc(ncepdp)
integer          icetsm(ncesmp), itypsm(ncesmp,nvar)

double precision dt(ncelet), rtpa(ncelet,*)
double precision propce(ncelet,*)
double precision propfa(nfac,*), propfb(nfabor,*)
double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
double precision crvexp(3,ncelet), crvimp(3,3,ncelet)

! Local variables

character*80     chaine
integer          iel, ipcrom, ipp

integer, allocatable, dimension(:) :: lstelt
!===============================================================================
!===============================================================================
double precision ubulk, ubulka, flux, surf, surfx, surfy, surfz, xrel                 
integer ifac,iifac
!===============================================================================

! Initialisation
if(ntcabs.eq.ntpabs-1) then
   ckp = 0.d0
   ubulk = uref
endif
ubulka = ubulk
flux = 0.d0
surf = 0.d0
! relaxation factor
xrel = 0.9d0
! loop on all faces and find the ones that are at x=0
! compute the total surface and the total mass flux

do ifac = 1, nfac
   if(abs(cdgfac(1,ifac)-0.d0).lt.1.d-4) then
      surfx = surfac(1,ifac)
      surfy = surfac(2,ifac)
      surfz = surfac(3,ifac)
      surf = surf + sqrt(surfx**2 + surfy**2 + surfz**2)
      
      iifac = iifac + 1
      flux = flux + propfa(ifac,ipprof(ifluma(iu)))
   endif
enddo
! for parallel calculations, sum on all processors

if(irangp.ge.0)then
   call parsom(flux)
   call parsom(surf)
   call parcpt(iifac)
endif

! obtain bulk velocity by dividing flux by area
ubulk = abs(flux) / surf
! correct the source term with the difference between
! the reference and the actual.
ckp = xrel*ckp + (1.d0-xrel)*(uref - ubulk)
!   write(nfecra, *) "Total number of faces at inlet:", iifac
!   write(nfecra, *) "Total surface at inlet:", surf
write(nfecra, *) "Bulk velocity at time step N:", ubulk
write(nfecra, *) "Bulk velocity at time step N-1:", ubulka
write(nfecra, *) "Reference bulk velocity:", uref
write(nfecra, *) "Relaxation factor:", xrel
write(nfecra, *) "Total correction term:",ckp

if (SGSMODEL.eq.0.or.SGSMODEL.eq.1) then
   do iel = 1, ncel
      crvexp(1,iel) = volume(iel)*ckp
   enddo
else if (SGSMODEL.eq.2) then
   do iel = 1, ncel
      crvexp(1,iel) = volume(iel)*ckp-volume(iel)*Fsgs(iel,1)
      crvexp(2,iel) = -volume(iel)*Fsgs(iel,2)
      crvexp(3,iel) = -volume(iel)*Fsgs(iel,3)
   enddo
else
write(nfecra,*) 'Source term not implemented'
endif
!--------
! Formats
!--------

 1000 format(' User source termes for variable ',A8,/)

!----
! End
!----

! Deallocate the temporary array
!deallocate(lstelt)

return
end subroutine ustsnv
